import pandas as pd
import sqlite3
import subprocess

def clean_old_databases():
    subprocess.run(["sh", "-c", f"rm /tmp/xapp_db*"])

def get_database_path():

    database_name = subprocess.check_output(['find', '/tmp', '-name', 'xapp_*', '-not', '-name', '*shm', '-not', '-name', '*wal', '-exec', 'basename', '{}', ';'])
    database_name = database_name.decode('utf-8').strip()
    database_path = r"{}".format("/tmp/"+str(database_name))

    return database_path

def create_connection(database_path):

    connection = None
    try:
        connection = sqlite3.connect(database_path)
    except sqlite3.Error as e:
        print(e)

    return connection

def select_mac_kpis(connection):

    cursor = connection.cursor()
    cursor.execute("SELECT tstamp, ngran_node, mcc, mnc, mnc_digit_len, nb_id, cu_du_id, frame, slot, dl_aggr_tbs, ul_aggr_tbs, dl_aggr_bytes_sdus, ul_aggr_bytes_sdus, dl_curr_tbs, ul_curr_tbs, dl_sched_rb, ul_sched_rb, pusch_snr, pucch_snr, rnti, dl_aggr_prb, ul_aggr_prb, dl_aggr_sdus, ul_aggr_sdus, dl_aggr_retx_prb, ul_aggr_retx_prb, wb_cqi, dl_mcs1, ul_mcs1, dl_mcs2, ul_mcs2, phr, bsr, dl_bler, ul_bler, dl_num_harq, dl_harq_round0, dl_harq_round1, dl_harq_round2, dl_harq_round3, dlsch_errors, ul_num_harq, ul_harq_round0, ul_harq_round1, ul_harq_round2, ul_harq_round3, ulsch_errors FROM MAC_UE")

    rows = cursor.fetchall()

    return rows

def select_metric_per_id_sample(connection, metric, id_sample):

    rows = pd.read_sql_query('SELECT ' + metric + ' FROM mac_kpis WHERE id_sample = ?', connection, params=[id_sample])

    return rows

def create_mac_kpis_table(connection):

    cursor = connection.cursor()
    cursor.execute('''CREATE TABLE IF NOT EXISTS mac_kpis
                           (tstamp text, ngran_node text, mcc text, mnc text, mnc_digit_len text, nb_id text, cu_du_id text, frame text, slot text, dl_aggr_tbs text, ul_aggr_tbs text, dl_aggr_bytes_sdus text, ul_aggr_bytes_sdus text, dl_curr_tbs text, ul_curr_tbs text, dl_sched_rb text, ul_sched_rb text, pusch_snr text, pucch_snr text, rnti text, dl_aggr_prb text, ul_aggr_prb text, dl_aggr_sdus text, ul_aggr_sdus text, dl_aggr_retx_prb text, ul_aggr_retx_prb text, wb_cqi text, dl_mcs1 text, ul_mcs1 text, dl_mcs2 text, ul_mcs2 text, phr text, bsr text, dl_bler text, ul_bler text, dl_num_harq text, dl_harq_round0 text, dl_harq_round1 text, dl_harq_round2 text, dl_harq_round3 text, dlsch_errors text, ul_num_harq text, ul_harq_round0 text, ul_harq_round1 text, ul_harq_round2 text, ul_harq_round3 text, ulsch_errors text, id_sample, sampling_frequency )''')

def save_mac_kpis_database(connection, data):

    cursor = connection.cursor()
    cursor.executemany("INSERT INTO mac_kpis VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", data)
    connection.commit()