import subprocess
from requests import request
from multiprocessing import Process
from xapp_mac_moni import generate_mac_kpi_samples
from db_xapp_framework import *

def start_measurement(sampling_time, measurement_time):
    measure = Process(target=generate_mac_kpi_samples, args=(sampling_time, measurement_time))
    measure.start()
    measure.join()

def load_mac_kpis_samples(id_current_sample, sampling_time):
    # Get results from database (with all KPIs)
    mac_kpis_sample = []
    measure_database_path = get_database_path()
    measure_database = create_connection(measure_database_path)
    mac_kpis_sample = select_mac_kpis(measure_database)
    mac_kpis_sample = [sample + (id_current_sample, sampling_time) for sample in mac_kpis_sample]

    return mac_kpis_sample

def init_mac_kpis_database(mac_kpis_database_path):
    # Create a new database to store only metrics of interest  
    mac_kpis_database = create_connection(mac_kpis_database_path)
    create_mac_kpis_table(mac_kpis_database)

    return mac_kpis_database

def verify_alarm_state(delta_percentage):

    if delta_percentage < 0.05:
        return '10'
    elif delta_percentage < 0.1:
        return '5'
    elif delta_percentage < 0.2:
        return '2'
    else:
        return '1'

#def update_dashboard_data():



if __name__ == "__main__":
    dict_sampling_times = {'10': 10, '5': 5, '2': 2, '1': 1}
    key_sampling_time = '1'
    sampling_time = dict_sampling_times.get(key_sampling_time)
    
    # Parameters
    total_measurement_time = 5
    measurement_time = 1
    mac_kpis_database_path = './results_mac_kpis_adaptive.sqlite'
    metric = "pusch_snr"

    mac_kpis_database = init_mac_kpis_database(mac_kpis_database_path)

    mac_kpis_sample = []
    id_current_sample = 0
    id_last_sample = 0
    current_measurement_time = 0

    while(current_measurement_time < total_measurement_time):
        clean_old_databases()

        start_measurement(sampling_time, measurement_time)

        mac_kpis_samples = load_mac_kpis_samples(id_current_sample, sampling_time)

        save_mac_kpis_database(mac_kpis_database, mac_kpis_samples)

        current_metric_samples = select_metric_per_id_sample(mac_kpis_database, metric, id_current_sample).astype(float)

        current_metric_mean = current_metric_samples.mean()

        last_metric_samples = select_metric_per_id_sample(mac_kpis_database, metric, id_last_sample).astype(float)

        last_metric_mean = last_metric_samples.mean()

        delta_percentage = (abs(current_metric_mean - last_metric_mean)/(last_metric_mean))[0]
        
        data_dict = {"id_sample": id_current_sample, "sampling_time": sampling_time, metric: current_metric_mean[0], "delta_percentage": round(delta_percentage*100,2)}

        #request(url=f'http://172.31.0.173:6662/updateData', method='POST', data=str(data_dict))

        key_sampling_time = verify_alarm_state(delta_percentage)

        sampling_time = dict_sampling_times.get(key_sampling_time)

        current_measurement_time += measurement_time
        
        if id_current_sample != 0:
            id_last_sample += 1
        
        id_current_sample += 1
         

