import xapp_sdk as ric
import time
import os
import pdb

####################
#### MAC INDICATION CALLBACK
####################

#  MACCallback class is defined and derived from C++ class mac_cb
class MACCallback(ric.mac_cb):
    # Define Python class 'constructor'
    def __init__(self):
        # Call C++ base class constructor
        ric.mac_cb.__init__(self)
    # Override C++ method: virtual void handle(swig_mac_ind_msg_t a) = 0;
    def handle(self, ind):
        # Print swig_mac_ind_msg_t
        if len(ind.ue_stats) > 0:
            t_now = time.time_ns() / 1000.0
            t_mac = ind.tstamp / 1.0
            t_diff = t_now - t_mac
            print('MAC Indication tstamp = ' + str(t_mac) + ' diff = ' + str(t_diff))
            #print('MAC rnti = ' + str(ind.ue_stats[0].rnti))
        

def get_sampling_time(sampling_time):

    if(sampling_time == 1):
        sampling_time_ms = ric.Interval_ms_1
    elif(sampling_time == 2):
        sampling_time_ms = ric.Interval_ms_2         
    elif(sampling_time == 5):
        sampling_time_ms = ric.Interval_ms_5
    else:
        sampling_time_ms = ric.Interval_ms_10

    return sampling_time_ms

def generate_mac_kpi_samples(sampling_time=1, measurement_time=10):        
    
    sampling_time_ms = get_sampling_time(sampling_time)
    
    ####################
    ####  GENERAL 
    ####################

    ric.init()

    conn = ric.conn_e2_nodes()
    assert(len(conn) > 0)
    for i in range(0, len(conn)):
        print("Global E2 Node [" + str(i) + "]: PLMN MCC = " + str(conn[i].id.plmn.mcc))
        print("Global E2 Node [" + str(i) + "]: PLMN MNC = " + str(conn[i].id.plmn.mnc))

    ####################
    #### MAC INDICATION
    ####################

    mac_hndlr = []
    for i in range(0, len(conn)):
        mac_cb = MACCallback()
        hndlr = ric.report_mac_sm(conn[i].id, sampling_time_ms, mac_cb)
        mac_hndlr.append(hndlr)     
        time.sleep(1)

    time.sleep(measurement_time)

    ### End

    for i in range(0, len(mac_hndlr)):
        ric.rm_report_mac_sm(mac_hndlr[i])

    # Avoid deadlock. ToDo revise architecture 
    while ric.try_stop == 0:
        time.sleep(1)
